/**
 * pb-commons
 * copyright PowerBEANS Co., Ltd.
 * create date 2006/10/05
 * amonden
 */
package jp.co.powerbeans.common.dict;

import java.util.MissingResourceException;

/**
 * 辞書翻訳クラス. dict.properties に基づいて変換を行う
 * 
 * @author amonden
 * @create date 2006/10/05
 */
public final class Dict {

	/**
	 * 
	 */
	private static final String EDIT = ".edit";

	/**
	 * 
	 */
	private static final String ORDER = ".order";

	/**
	 * プロパティファイル名
	 */
	private static final String PROP = "dict";

	/**
	 * dict.properties のリソースバンドル
	 */
	private static java.util.ResourceBundle bundle;
	static {
		try {
			bundle = java.util.ResourceBundle.getBundle(PROP);
		} catch (java.util.MissingResourceException e) {
			System.err.println("[PBSWT] can't open " + PROP
					+ ".properties. please create \"" + PROP
					+ ".properties\" in source root.");
		} catch (RuntimeException e) {
			e.printStackTrace();
		}
	}

	/**
	 * name を翻訳する
	 * 
	 * @param name
	 * @return 翻訳結果
	 */
	public static String trans(String name) {
		if (!bundleCheck()) {
			return "";
		}
		try {
			return bundle.getString(name);
		} catch (MissingResourceException e) {
			System.err.println("[PBSWT] can't translate from " + name
					+ ". please add value to " + PROP + ".properties (ex: "
					+ name + "=DisplayLabel");
			return name;
		} catch (RuntimeException e) {
			throw e;
		}
	}

	/**
	 * 存在するはずの項目数 を指定して name を翻訳する
	 * 
	 * @param name
	 * @param itemnum
	 *            項目数
	 * @return 翻訳結果
	 */
	public static String[] transMulti(String name, String[] default_vals) {
		int itemnum = default_vals.length;
		if (!bundleCheck()) {
			return default_vals;
		}
		try {
			String[] items = bundle.getString(name).split(",");
			if (items.length != itemnum) {
				System.err.println("[PBSWT] can't match item num " + name
						+ ". please add value to " + PROP + ".properties (ex: "
						+ name + "=label1,label2,..");
				return default_vals;
			}
			return items;
		} catch (MissingResourceException e) {
			System.err.println("[PBSWT] can't translate from " + name
					+ ". please add value to " + PROP + ".properties (ex: "
					+ name + "=DisplayLabel");
			return default_vals;
		} catch (RuntimeException e) {
			throw e;
		}
	}

	/**
	 * dict.properties ファイルが存在するかどうか
	 * 
	 * @return true 存在する, false 存在しない
	 */
	private static boolean bundleCheck() {
		if (bundle == null) {
			System.err.println("[PBSWT] can't open resouce. please create \""
					+ PROP + ".properties\" in source root.");
			return false;
		}
		return true;
	}

	/**
	 * dict.propertiesから model_name(or simple_model_name) の name1カラム と name2カラム
	 * の順序を取得
	 * 
	 * @param model_name
	 *            モデル名
	 * @param simple_model_name
	 *            省略モデル名.モデル名が存在しない場合利用する
	 * @param name1
	 *            プロパティ1
	 * @param name2
	 *            プロパティ2
	 * @return 1 name1 > name2, -1 name1 < name2, 0 一緒
	 */
	public static int compare(String model_name, String simple_model_name,
			String name1, String name2) {
		String order = "";
		String key = "";
		try {
			key = model_name + ORDER;
			order = bundle.getString(key);
		} catch (RuntimeException e) {
			// 存在しなくてもいいので無視
			try {
				key = simple_model_name + ORDER;
				order = bundle.getString(key);
			} catch (RuntimeException e1) {
				System.err.println("[PBSWT] can't get order of " + key
						+ ". please set order to " + PROP + ".properties (ex: "
						+ key + "=name,old");
			}
		}

		// 各プロパティのインデックスを取得
		int name1ind = order.indexOf(name1);
		int name2ind = order.indexOf(name2);
		// 両方とも -1 の場合はどちらも非表示
		if (name1ind == -1 && name2ind == -1) {
//			System.err.println("[PBSWT] can't get order of " + key + "."
//					+ name1 + "," + name2 + " . please set order to " + PROP
//					+ ".properties (ex: " + key + "=name,old");
			return 0;
		} else if (name1ind == -1) {
			return 1;
		} else if (name2ind == -1) {
			return -1;
		}
		return name1ind > name2ind ? 1 : -1;
	}

	/**
	 * @param key
	 * @return
	 */
	public static boolean hasKey(String key) {
		try {
			bundle.getString(key);
			return true;
		} catch (RuntimeException e) {
			return false;
		}
	}

	/**
	 * master_cls の order の列リストに name が存在するかどうか
	 * 
	 * @param master_cls マスタクラス名
	 * @param name プロパティ名
	 * @return true 存在する, false 存在しない
	 */
	public static boolean hasOrder(Class master_cls, String name) {

		String key = master_cls.getSimpleName() + ORDER;
		try {
			String val = bundle.getString(key);
			return val != null && val.indexOf(name) != -1;
		} catch (MissingResourceException e) {
			System.err.println("[PBSWT] can't get key " + key  + " from " + PROP
					+ ".properties. please set order to " + PROP
					+ ".properties (ex: " + key + "=name,old");
			return false;
		}
	}
	
	/**
	 * master_cls の edit の列リストに name が存在するかどうか
	 * 
	 * @param master_cls マスタクラス名
	 * @param name プロパティ名
	 * @return true 存在する, false 存在しない
	 */
	public static boolean hasEdit(Class master_cls, String name) {

		String val = getEditColumnValue(master_cls);
		return val != null && val.indexOf(name) != -1;
	}

	/**
	 * @param master_cls 
	 * @param name
	 * @return
	 */
	private static String getEditColumnValue(Class master_cls) {
		String key = master_cls.getSimpleName() + EDIT;
		try {
			String val = bundle.getString(key);
			return val;
		} catch (MissingResourceException e) {
			System.err.println("[PBSWT] can't get key " + key  + " from " + PROP
					+ ".properties. please set order to " + PROP
					+ ".properties (ex: " + key + "=name,old");
			return null;
		}
	}

	/**
	 * master_cls の edit のカラムリストを返す
	 * @param master_cls マスタクラス名
	 * @return カラムリスト
	 */	
	public static String[] getEditColumns(Class master_cls) {
		
		String val = getEditColumnValue(master_cls);
		return val != null ? val.split(",") : new String[]{};
	}

}
