/**
 * $Id$
 */
package jp.co.powerbeans.kaishuj.logic;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import jp.co.powerbeans.common.db.DBManager;
import jp.co.powerbeans.kaishuj.logic.parser.BlockParser;
import jp.co.powerbeans.kaishuj.logic.parser.BlockParserFactory;
import jp.co.powerbeans.kaishuj.models.ClassInfo;
import jp.co.powerbeans.kaishuj.models.JavadocUrl;

/**
 * JavadocRoundMgr
 * 
 * @author amonden
 * @create 2006/12/21
 */
public class JavadocRoundMgr {

  public enum ClassUrlType {
    Interface, Class
  }

  /**
   * ClassUrl
   * 
   * @author amonden
   * @create 2006/12/21
   */
  public class ClassUrl implements Serializable {

    private String url;

    private String clsName;

    private ClassUrlType type;

    /**
     * @param url
     * @param cls_name
     * @param type
     */
    public ClassUrl(String url, String cls_name, ClassUrlType type) {
      this.url = url;
      this.clsName = cls_name;
      this.type = type;
    }

    /**
     * @return clsName を戻します。
     */
    public String getClsName() {
      return this.clsName;
    }

    /**
     * @return url を戻します。
     */
    public String getUrl() {
      return this.url;
    }

    /**
     * @return type を戻します。
     */
    public ClassUrlType getType() {
      return this.type;
    }

  }

  private JavadocUrl javadocUrl;
  private JavadocRoundLogListener logListener;
  private String targetCls;

  /**
   * @param javadocUrl
   * @param target_cls 
   */
  public JavadocRoundMgr(JavadocUrl javadocUrl, JavadocRoundLogListener log_listener, String target_cls) {
    this.javadocUrl = javadocUrl;
    this.logListener = log_listener;
    this.targetCls = target_cls;
  }

  /**
   * 
   */
  public int start() {
    // フレームページを解析
    String root_url = this.javadocUrl.getUrl();
    String base_url = getBasePath(root_url);
    String allclass_url = createAllClassUrl(base_url);

    // 全クラスとURLのセットを取得
    ClassUrl[] class_urls = roundToClassUrl(allclass_url);

    this.logListener.output("start get " + class_urls.length + " items.");
    
    // 全セットを解析して直接DBに格納
    for (int i = 0; i < class_urls.length; i++) {
      ClassInfo class_info = parseClassInfo(class_urls[i]);
      this.logListener.output((i + 1) + "/" + class_urls.length + " " + class_info.getClassName() + " from " + class_info.getClassJavadocUrl());
    }

    this.logListener.output("done.");
    
    return class_urls.length;
  }

  /**
   * cls_url の url からクラス詳細を取得
   * 
   * @param cls_url
   * @return
   */
  private ClassInfo parseClassInfo(ClassUrl cls_url) {
    ClassInfo cls_info = new ClassInfo();
    cls_info.setClassName(cls_url.getClsName());
    cls_info.setClassJavadocUrl(cls_url.getUrl());
    cls_info.setJavadocUrlId(this.javadocUrl.getJavadocUrlId());

    // WEBから取得して解析
    try {
      URL url = new URL(cls_url.getUrl());

      // 入力ストリームを生成
      BufferedReader in = null;
      if (this.javadocUrl.getEncode() != null) {
       in = new BufferedReader(new InputStreamReader(url
          .openStream(), this.javadocUrl.getEncode()));
      } else {
        in = new BufferedReader(new InputStreamReader(url.openStream()));
      }

      // 一行ずつ読み込みます
      String line = null;
      BlockParser block_parser = BlockParserFactory.createInstance(null);
      while ((line = in.readLine()) != null) {
        // block チェック
        if (BlockParserFactory.isNewBlock(line)) {
          block_parser.finish(cls_info);
          String block_name = BlockParserFactory.getBlockNameFromSrcHtml(line);
          // 次のブロックを解析
          block_parser = BlockParserFactory.createInstance(block_name);
        }

        // 1行ずつブロックを解析
//        if (this.javadocUrl.getEncode() != null) {
//          line = new String(line.getBytes(), this.javadocUrl.getEncode());
//        }

        block_parser.parseLine(line);
      }
    } catch (IOException e) {
      throw new RuntimeException(e);
    }

    return cls_info;
  }

  private String getBasePath(String root_url) {
    return root_url.endsWith("/") ? root_url : root_url.substring(0, root_url
        .lastIndexOf("/") + 1);
  }

  /**
   * HTMLから ClassUrl を生成. HTMLソース例(
   * 
   * <pre>
   * &lt;A HREF=&quot;javax/swing/AbstractAction.html&quot; TARGET=&quot;classFrame&quot;&gt;AbstractAction&lt;/A&gt;
   * </pre>)
   * 
   * @param allclass_page
   * @return
   */
  private ClassUrl parseClassUrl(String base_url, String allclass_page) {
    String allclass_page_up = allclass_page.toUpperCase();
    int left = allclass_page_up.indexOf("<A HREF=\"");
    if (left == -1) {
      return null;
    }
    String sub_url = allclass_page.substring(left + 9, allclass_page.indexOf(
        "\"", left + 10));
    int cls_top = allclass_page.indexOf("\">", left) + 2;
    String class_name = allclass_page.substring(cls_top, allclass_page_up
        .indexOf("</A>", cls_top));

    // インターフェースチェック( <I> </I> で囲まれているか)
    boolean if_flg = (class_name.toUpperCase().startsWith("<I>"));
    ClassUrlType type = if_flg ? ClassUrlType.Interface : ClassUrlType.Class;
    class_name = class_name.replaceAll("/", "").replaceAll("<I>", "")
        .replaceAll("<i>", "");

    return new ClassUrl(base_url + sub_url, class_name, type);
  }

  private String createAllClassUrl(String base_url) {
    // '/' 移行を削除
    return base_url + "allclasses-frame.html";
  }

  /**
   * @param url
   *          巡回URL
   * @return 巡回結果
   */
  private ClassUrl[] roundToClassUrl(String src_url) {

    List list = new ArrayList();
    try {
      URL url = new URL(src_url);
      String base_url = getBasePath(src_url);

      // 入力ストリームを生成
      BufferedReader in = new BufferedReader(new InputStreamReader(url
          .openStream()));

      // 一行ずつ読み込みます
      String line = null;
      while ((line = in.readLine()) != null) {
        // charset 解析
        if (this.javadocUrl.getEncode() == null) {
          int top = -1;
          if ((top = line.toUpperCase().indexOf("CHARSET=")) != -1) {
            top += 8;
            try {
              String lines = line.replace('\'', '"');
              this.javadocUrl.setEncode(lines.substring(top, lines.indexOf(
                  "\"", top)));
              DBManager.update(this.javadocUrl, new String[] { "encode" });
            } catch (java.lang.StringIndexOutOfBoundsException e) {
            }
          }
        }

        // 対象クラスと一致するか？
        {
          String line_package = line.replaceAll("/",".");
          if (!line_package.matches("^.*" + this.targetCls + ".*$")) {
            // 一致しない場合は無視
            continue;
          }
        }
        // クラス解析
        ClassUrl class_url = parseClassUrl(base_url, line);
        if (class_url != null) {
          list.add(class_url);
        }
      }
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
    return (ClassUrl[]) list.toArray(new ClassUrl[list.size()]);
  }
}
